from abuseipdb import AbuseIPDB

with AbuseIPDB() as client:
    print(client.reports("8.8.8.8", page=1, per_page=25))
    print(client.blacklist(confidence_minimum=90, ip_version=6))
    print(client.blacklist(confidence_minimum=90, plaintext=True).addresses)
    print(client.check_block("2001:db8::/64"))
