from abuseipdb import AbuseIPDB

# These calls alter reports on your AbuseIPDB account. Replace documentation IPs intentionally.
csv_contents = "IP,Categories,ReportDate,Comment\n203.0.113.10,18,,Brute force\n"
with AbuseIPDB() as client:
    print(client.bulk_report(csv_contents))
    print(client.clear_address("203.0.113.10"))
